package OESSDatabaseTester;

use strict;
use FindBin;
use GRNOC::Config;
use Data::Dumper;
use DBI;

$ENV{"PATH"} = "";

sub getConfigFilePath {
    my $cwd = $FindBin::Bin;
    $cwd =~ /(.*)/;
    $cwd = $1;
    my $line;
    my $database_config = "$cwd/conf/database.xml";
   
    return $database_config;
}

sub getConfig { 
    my $cwd = $FindBin::Bin;
    $cwd =~ /(.*)/;
    $cwd = $1;
    my $cfg                 = GRNOC::Config->new(config_file => "$cwd/conf/database.xml");
    my $user                = $cfg->get('/config/credentials[1]/@username')->[0];
    my $pass                = $cfg->get('/config/credentials[1]/@password')->[0];
    my $db                  = $cfg->get('/config/credentials[1]/@database')->[0];
    my $result = {"user"   => $user,
                  "pass"   => $pass,
                  "db"     => $db
    };

    return $result;
}

sub resetSNAPPDB {

    my $creds = &getConfig();
    #drop the snapp-test DB if it exists, and then create it.
    my %attr = (PrintError => 0, RaiseError => 0);
    my $dbh = DBI->connect("DBI:mysql:dbname=;host=localhost;port=6633",$creds->{'user'},$creds->{'pass'},\%attr);
    $dbh->do("create database " . 'snapp_test' );
    $dbh->do("set foreign_key_checks = 0");
    
    my $cwd = $FindBin::Bin;
    $cwd =~ /(.*)/;
    $cwd = $1;
    my $command = "/usr/bin/mysql -u $creds->{'user'} --password=$creds->{'pass'} snapp_test < $cwd/conf/snapp_known_state.sql";
    if (system($command)){
        return 0;
    }

    #this goes and sets up the snapp directory for the tests

    $dbh->do("use snapp_test");
    $dbh->do("UPDATE global SET value='$cwd/conf/SNMP/snapp/db/' WHERE name = 'rrddir'");

    my $sth = $dbh->prepare("UPDATE global SET value ='$cwd/conf/SNMP/snapp/db/' WHERE name = 'rrddir'");
    $sth->execute();
    $sth = $dbh->prepare('SELECT value FROM global');
    $sth->execute();
    my $result = $sth->fetchrow_hashref();
   
    $dbh->do("set foreign_key_checks = 1"); 

    return 1;
}

sub resetOESSDB {
    my $creds = &getConfig();
    #drop the oess-test DB if it exists, and then create it
    my %attr = (PrintError => 0, RaiseError => 0);
    my $dbh = DBI->connect("DBI:mysql:dbname=;host=localhost;port=6633",$creds->{'user'},$creds->{'pass'},\%attr);
    $dbh->do("create database " . $creds->{'db'});
    $dbh->do("set foreign_key_checks = 0");

   
    my $cwd = $FindBin::Bin;
    $cwd =~ /(.*)/;
    $cwd = $1;
    my $command = "/usr/bin/mysql -u $creds->{'user'} --password=$creds->{'pass'} $creds->{'db'} < $cwd/conf/oess_known_state.sql";
    if (system($command)){
        return 0;
    }

    $dbh->do("set foreign_key_checks = 1");
    return 1;
}

# convenience method for bumping/saging those pesky workgroup limits
sub workgroupLimits {
    my %args = @_;
    my $workgroup_id = $args{'workgroup_id'};
    my $db           = $args{'db'};
    my $circuit_num  = $args{'circuit_num'};
    my $op           = $args{'op'} || '+';

    # get workgroup
    my $workgroup = $db->_execute_query(
        'SELECT * from workgroup where workgroup_id = ?', 
        [$workgroup_id]
    )->[0];

    # +/-= any defined limit variables
    my $mac_addr_per_end = (defined($args{'mac_addr_endpoint_num'}))
        ? eval($workgroup->{'max_mac_address_per_end'}." $op ".$args{'mac_addr_endpoint_num'})
        : $workgroup->{'max_mac_address_per_end'};

    my $max_ckts = (defined($args{'circuit_num'}))
        ? eval($workgroup->{'max_circuits'}." $op ".$args{'circuit_num'})
        : $workgroup->{'max_circuits'};
        
    my $max_ckt_endpoints = (defined($args{'circuit_endpoints_num'}))
        ? eval($workgroup->{'max_circuit_endpoints'}." $op ".$args{'circuit_endpoints_num'})
        : $workgroup->{'max_circuit_endpoints'};

    # update that stuff
    my $res = $db->update_workgroup(
        workgroup_id             => $workgroup->{'workgroup_id'},
        name                     => $workgroup->{'name'},
        external_id              => $workgroup->{'external_id'},
        max_mac_address_per_end  => $mac_addr_per_end,
        max_circuits             => $max_ckts,
        max_circuit_endpoints    => $max_ckt_endpoints 
    );

    warn Dumper($res);
}


1;
